package com.persagy.dc.mybatis.handler;

import com.persagy.dc.common.context.AppContext;
import com.persagy.dc.common.lang.PsDateTime;
import com.persagy.dc.common.model.entity.AuditableEntity;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.SqlCommandType;
import org.apache.ibatis.plugin.Interceptor;
import org.apache.ibatis.plugin.Intercepts;
import org.apache.ibatis.plugin.Invocation;
import org.apache.ibatis.plugin.Signature;
import org.springframework.stereotype.Component;

/**
 * 审计信息拦截器
 * 保存前审计信息赋值公共处理
 * @author Charlie Yu
 * @date 2021-06-16
 */
@Slf4j
@Component
@Intercepts({@Signature(type = Executor.class, method = "update", args = {MappedStatement.class, Object.class})})
public class AuditInfoHandler implements Interceptor {

    @Override
    public Object intercept(Invocation invocation) throws Throwable {
        MappedStatement mappedStatement = (MappedStatement) invocation.getArgs()[0];
        SqlCommandType sqlCommandType = mappedStatement.getSqlCommandType();
        // 新增/修改时才需要处理
        if(SqlCommandType.INSERT != sqlCommandType && SqlCommandType.UPDATE != sqlCommandType) {
            return invocation.proceed();
        }
        Object parameter = invocation.getArgs()[1];
        // 只处理审计对象
        if (parameter == null || !(parameter instanceof AuditableEntity)) {
            return invocation.proceed();
        }
        // 审计信息默认值
        AuditableEntity entity = (AuditableEntity) parameter;
        if (SqlCommandType.INSERT == sqlCommandType) {
            entity.setCreator(AppContext.getContext().getAccountId());
            entity.setCreationTime(new PsDateTime());
        } else {
            entity.setModifier(AppContext.getContext().getAccountId());
            entity.setModifiedTime(new PsDateTime());
        }
        return invocation.proceed();
    }
}
